@@ -5,7 +5,6 @@
 import logging
 import os.path
 import pandas as pd
-import statistics
 
 from jcast import params
 
@@ -164,13 +163,10 @@ class Junction(object):
         self.sjc_s1 = kwargs['sjc_s1']
         self.sjc_s2 = kwargs['sjc_s2']
 
-
         self.tx1 = None
         self.tx0 = None
         self.phase = None
         self.num_start_codons = 0
-        # self.min_read_count = 0
-
 
         self.logger = logging.getLogger('jcast.junction')
 
@@ -186,9 +182,9 @@ class Junction(object):
                + self.junction_type + ' ' + self.gene_symbol + ' ' + self.name
 
     @property
-    def min_read_count(self):
+    def sum_sjc(self):
         """
-        Returns the minimum read count in the junction. if rMATS was run with one technical replicate,
+        Returns the sum of all SJCs in the junction. if rMATS was run with one technical replicate,
         the count field is an int, otherwise it is a list. Currently this takes the skipped junction count (SJC)
         as filtering criterion because the majority of translatable events are probably SE (skipped exon).
         Essentially this filters out alternative junctions that are very rarely skipped (high inclusion
@@ -197,14 +193,14 @@ class Junction(object):
         :return:
         """
         try:
-            mean_count_sample1 = int(statistics.mean([int(x) for x in (str(self.sjc_s1).split(sep=','))]))
-            mean_count_sample2 = int(statistics.mean([int(x) for x in (str(self.sjc_s2).split(sep=','))]))
+            sum_count_sample1 = int(sum([int(x) for x in (str(self.sjc_s1).split(sep=','))]))
+            sum_count_sample2 = int(sum([int(x) for x in (str(self.sjc_s2).split(sep=','))]))
 
         except ValueError:
-            mean_count_sample1 = 0
-            mean_count_sample2 = 0
+            sum_count_sample1 = 0
+            sum_count_sample2 = 0
 
-        return min([mean_count_sample1, mean_count_sample2])
+        return sum_count_sample1 + sum_count_sample2
 
     def _get_translated_region(self,
                                gtf,
