@@ -8,12 +8,16 @@ import logging
 from functools import partial
 
 import tqdm
+from sklearn.mixture import GaussianMixture
+from sklearn.preprocessing import PowerTransformer
+import numpy as np
+import matplotlib.pyplot as plt
+import scipy.stats as stats
 
 from jcast import params, fates
 from jcast.junctions import Junction, RmatsResults
 from jcast.annots import ReadAnnotations, ReadGenome
 from jcast.sequences import Sequence
-
 from jcast import __version__
 
 
@@ -62,6 +66,8 @@ def runjcast(args):
     assert os.path.exists(os.path.join(args.rmats_folder, 'MXE.MATS.JC.txt')), 'rMATS files not found, check directory.'
     rmats_results = RmatsResults(rmats_dir=args.rmats_folder)
 
+    # Model read count cutoff
+
     #
     # Read the gtf file using the gtfpase package.
     # Then write as a pandas data frame.
@@ -74,6 +80,76 @@ def runjcast(args):
     #
     genome = ReadGenome(args.genome)
 
+    #
+    # Model read count cutoff.
+    # TODO: move this to a separate class
+    #
+    if args.model:
+
+        main_log.info('The -m flag is set. The modeled read count will override -r --read values.')
+
+        tot = rmats_results.rmats_mxe.append(rmats_results.rmats_se).append(rmats_results.rmats_ri).append(
+            rmats_results.rmats_a5ss).append(rmats_results.rmats_a3ss).copy()
+        junctions = [Junction(**tot.iloc[i].to_dict()) for i in range(len(tot))]
+
+        # Array of junction sum read counts
+        matrix_X = np.array([[j.sum_sjc + 1] for j in junctions])
+        pt = PowerTransformer(method='box-cox')
+        pt.fit(matrix_X)
+        matrix_X_trans = pt.transform(matrix_X)
+
+        gmm = GaussianMixture(n_components=2,
+                              covariance_type='diag',
+                              random_state=1,
+                              ).fit(matrix_X_trans)
+
+        # Sort the classification components so the high-read distribution appears second
+        order = gmm.means_.argsort(axis=0)[:, 0]
+        gmm.means_ = gmm.means_[order]
+        gmm.covariances_ = gmm.covariances_[order]
+        gmm.weights_ = gmm.weights_[order]
+        gmm.precisions_ = gmm.precisions_[order]
+        gmm.precisions_cholesky_ = gmm.precisions_cholesky_[order]
+
+        weights = gmm.weights_
+        means = gmm.means_
+        covars = gmm.covariances_
+
+        # Get the decision boundary (minimum count required to be predicted as second predicted class)
+        # TODO: catch errors where model may fail and min_count remains 1
+        min_count = 1
+        # Loop only from 1 count to the mean of the second distribution
+        for i in range(1, round(pt.inverse_transform([gmm.means_[1]])[0][0])):
+            dist = gmm.predict(pt.transform(np.array([[i]])))
+            if dist == [1]:
+                min_count = i
+                break
+
+        # Plot out the model figure and decision boundary
+        fig, ax = plt.subplots(constrained_layout=True)
+        ax.hist(matrix_X_trans, bins=50, histtype='bar', density=True, ec='red', alpha=0.5)
+        plt.style.use('seaborn-white')
+        ax.set_xlabel('Box-Cox transformed total skipped junction counts')
+        ax.set_ylabel('Frequency')
+        ax.set_title('Skipped junction count distributions')
+        tcks, values = plt.xticks()
+        new_tcks = np.int_(np.sort(pt.inverse_transform(np.array([[t] for t in tcks])).ravel()))
+
+        # Second axis for untransformed reads
+        ax2 = ax.twiny()
+        ax2.set_xlim(ax.get_xlim())
+        ax2.set_xticks(tcks[1:-1])
+        ax2.set_xlabel('Untransformed total splice junction read counts')
+        ax2.set_xticklabels(new_tcks[1:-1])
+        ax2.axvline(linewidth=4, ls='--', color='blue', x=pt.transform(np.array([[min_count]])))
+
+        f_axis = matrix_X_trans.copy().ravel()
+        f_axis.sort()
+        ax.plot(f_axis, weights[0] * stats.norm.pdf(f_axis, means[0], np.sqrt(covars[0])).ravel(), c='red')
+        ax.plot(f_axis, weights[1] * stats.norm.pdf(f_axis, means[1], np.sqrt(covars[1])).ravel(), c='blue')
+
+        plt.savefig(fname=os.path.join(write_dir, 'model.png'))
+
     #
     # Main loop through every line of each of the five rMATS files to make junction object, then translate them
     #
@@ -91,6 +167,7 @@ def runjcast(args):
                                         genome=genome,
                                         args=args,
                                         write_dir=write_dir,
+                                        pred_bound=min_count,
                                         )
 
         #
@@ -135,22 +212,10 @@ def _translate_one(junction,
                    genome,
                    args,
                    write_dir,
-                    ):
+                   pred_bound,
+                   ):
     """ get coordinate and translate one junction; arguments are passed through partial from main"""
 
-    #
-    # filter by junction read counts - discard junction if the min read count is below threshold
-    #
-    if junction.min_read_count < args.read:
-        return fates.skipped_low
-
-    #
-    # discard junction if the corrected P value of this read count is < threshold
-    # this removes junctions that are inconsifound on both replicates.
-    #
-    if junction.fdr < args.pvalue:
-        return fates.skipped_low
-
     #
     # trim slice coordinates by translation starts and ends
     #
@@ -180,6 +245,37 @@ def _translate_one(junction,
     sequence.get_canonical_aa(gtf=gtf, genome_index=genome.genome)
     sequence.translate(use_phase=True)
 
+    #
+    # filter by junction read counts - discard junction if the min read count is below threshold
+    #
+
+    # If the -r argument is set directly and the -m flag is not, use the -r integer for count filtering
+    # If the -m flag is set, use the modeled count for filtering
+    if (not args.model and junction.sum_sjc < args.read) or (args.model and junction.sum_sjc < pred_bound):
+        #
+        # If the canonical flag is set, append the canonical
+        # Sp to the gene_canonical output even if none of the transcript slices are stitchable
+        # back to the canonical protein. This avoids not having any protein level representation
+        # of a gene potentially in the proteome.
+        #
+        if args.canonical:
+            sequence.write_canonical(outdir=write_dir)
+
+        return fates.skipped_low
+
+    #
+    # discard junction if the corrected P value of this read count is < threshold
+    # this removes junctions that are inconsistently found on both replicates.
+    #
+    q_lo, q_hi = args.qvalue
+    if not q_lo < junction.fdr < q_hi:
+
+        # Write canonical anyhow if the canonical flag is set.
+        if args.canonical:
+            sequence.write_canonical(outdir=write_dir)
+
+        return fates.skipped_low
+
     #
     # write the Tier 1 and Tier 2 results into fasta file
     #
@@ -316,30 +412,49 @@ def main():
     parser = argparse.ArgumentParser(description='jcast retrieves transcript splice junctions'
                                                  'and translates them into amino acid sequences')
 
-    parser.add_argument('rmats_folder', help='path to folder storing rMATS output')
-    parser.add_argument('gtf_file', help='path to Ensembl gtf file')
-    parser.add_argument('genome', help='path to genome file')
+    parser.add_argument('rmats_folder',
+                        help='path to folder storing rMATS output',
+                        )
+    parser.add_argument('gtf_file',
+                        help='path to Ensembl gtf file',
+                        )
+    parser.add_argument('genome',
+                        help='path to genome file',
+                        )
 
     # parser.add_argument('-n', '--num_threads', help='number of threads for concurrency [default: 6]',
     #                     default=6,
     #                     type=int)
 
-    parser.add_argument('-o', '--out', help='name of the output files [default: psq_out]',
+    parser.add_argument('-o', '--out',
+                        help='name of the output files [default: psq_out]',
                         default='out')
 
-    parser.add_argument('-r', '--read'
-                        , help='minimum read counts to consider [default: 1]',
+    parser.add_argument('-r', '--read',
+                        help='the lowest skipped junction read count for a junction to be translated [default: 1]',
                         default=1,
-                        type=int)
+                        type=int,
+                        )
+
+    parser.add_argument('-m', '--model',
+                        help='models junction read count cutoff using a Gaussian mixture model [default: False]',
+                        action='store_true',
+                        default=False,
+                        #type=bool,
+                        )
 
     parser.add_argument('-c', '--canonical', help='write out canonical protein sequence even if transcript'
-                                   'slices are untranslatable [default: True]',
-                        default=True,
-                        type=bool)
+                                   'slices are untranslatable [default: False]',
+                        default=False,
+                        action='store_true',
+                        # type=bool,
+                        )
 
-    parser.add_argument('-p', '--pvalue'
-                        , help='discard junctions with rMATS pvalue below this threshold [default: 0.01]',
-                        default=0.01,
+    parser.add_argument('-q', '--qvalue',
+                        help='take junctions with rMATS fdr within this threshold [default: 0 1]',
+                        metavar=('q_lo', 'q_hi'),
+                        nargs=2,
+                        default=[0, 1],
                         type=float)
 
     parser.set_defaults(func=runjcast)
