@@ -436,6 +436,7 @@ class Sequence(object):
             if ret.status_code == 200 and ret.text != '':
                 record = list(SeqIO.parse(StringIO(ret.text), 'fasta'))[0]
 
+                # TODO: Catch sqlite3.OperationalError if writing fails, such as in a remote volume.
                 cur.execute('''INSERT INTO sequences(id, seq) VALUES(:id, :seq)''',
                             {'id': cache, 'seq': record.format('fasta')})
                 con.commit()
@@ -574,7 +575,7 @@ class Sequence(object):
                                            self.j.alt1_ee,
                                            self.translated_strand,
                                            self.translated_phase,
-                                           self.j.min_read_count,
+                                           self.j.sum_sjc,
                                            suffix,
                                        ),
                                        annotations={'molecule_type': 'extended_protein'},
@@ -601,7 +602,7 @@ class Sequence(object):
                     self.j.alt1_ee,
                     self.translated_strand,
                     self.translated_phase,
-                    self.j.min_read_count,
+                    self.j.sum_sjc,
                     suffix,
                 )
                 h.write_seqrecord_to_fasta(stitched, outdir, suffix)
